/*
**	 +----------------------------------------------------------------+
**	 | show_startup:  check NW Interfaces to startup LoRa GW software |
**	 +----------------------------------------------------------------*
**	 
**	 Startup Program to check Network Connectivity and Display IP@
**	 added feature to query GPS-Daemon to replace Coordinates in APRS.conf
**	 before starting up iot4p-GW-Software
**	 
**	 additional interfaces can be added thru 'ret = show_IP( "wlan0" );'
**	 interface names can be retrieved by 'ifconfig (1)'
**	 
**	 needs additional libraries libgps, wiringPI and oled96
**	 complie:  make     (using supplied Makefile)
**	 	( cc show_startup.o get_gpsdata.o -loled96 -lm -lpthread 
**			-lwiringPi -lgps -lm -o show_startup )
**	 
**	 Author: OE1ACM / Bernd Gasser   lora-aprs.at   2020/05/03
**	 
*/


#include <stdint.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include "oled96.h"
#include <wiringPi.h>



#include <sys/types.h>
#include <sys/socket.h>
#include <sys/ioctl.h>
#include <netinet/in.h>
#include <net/if.h>
#include <arpa/inet.h>

int show_IP( char *interf );

extern unsigned char ucSmallFont[];
int	line=1;
int	interf_found = 0;     // Interfaces found 
int	attempt = 0;

int	ud=0, lr=0, ent=0;  // buttons

int	verbose = 0;


#define INPUT_UP 	27
#define INPUT_DWN 	28
#define	INPUT_ENT 	29
#define	INPUT_LEFT	24
#define	INPUT_RIGHT 	25

int main(int argc, char *argv[])
{
int 	i = 0;
int	ret = 0;
int	button_pressed = 0;

	set_buttons();

	i=oledInit(1, 0x3c, 0, 0); // for Raspberry Pi, use channel 1
	if (i == 0)
	{
		oledFill(0); // fill with black
		oledWriteString(0,0,"Starting ....",FONT_NORMAL);
		oledWriteString(0,3,"press Key ",FONT_SMALL);
		oledWriteString(0,4,"to set GPS-Data",FONT_SMALL);
		sleep(2);
		oledWriteString(0,6,"checking NW...",FONT_NORMAL);
		sleep(3);


		while ( interf_found < 1) {

			// check buttons to rebuild GPS Coordinates 
			button_pressed += check_buttons();

			// clear screen
			oledFill(0); // fill with black

			if ( button_pressed ) 	oledWriteString(0,0,"(GPS) Interfaces",FONT_NORMAL);
			else 			oledWriteString(0,0,"      Interfaces",FONT_NORMAL);

			interf_found = 0;
			line=1;
			line++;
			ret = show_IP( "wlan0" );
			ret = show_IP( "eth0" );
			ret = show_IP( "usb0" );
			//ret = show_IP( "lo" );                //  add aditional interfaces
			sleep(3);

			if ( interf_found == 0 ) {

				oledWriteString(0,4 , "no NW seen,retrying", FONT_SMALL);
				sleep(1);
			}

			if ( attempt++ >= 50 )  {
				
				oledWriteString(0,5 , "no WAN - exiting", FONT_NORMAL);
				exit(5);
			}
		}

		//oledWriteString(3,5,"Narrow Font (6x8)", FONT_SMALL);

		line=7;
 		oledWriteString(0,line, "ping APRS_IS_HOST ...", FONT_SMALL);
		sleep(3);
		system( "./check_WAN_connection");
		 oledWriteString(0,line, "WAN ok, got response", FONT_SMALL);
		//if( verbose) printf("Press ENTER to quit\n");
		//getchar();


		// check whether we want to replace GPS-Coordinates in  APRS.conf 

		if ( button_pressed ) ret=replace_GPS_Data(); 

		 oledWriteString(0,line, "starting GW-SW ...", FONT_NORMAL);

		sleep(5);
		oledShutdown();
	}
   exit(0);
} /* main() */

char	IP_interf[32];


int show_IP( char *interf )
{
 int fd;
 int ret = 0;
 struct ifreq ifr;

	memset(IP_interf,0,sizeof(IP_interf));

 	fd = socket(AF_INET, SOCK_DGRAM, 0);

 	/* I want to get an IPv4 IP address */
 	ifr.ifr_addr.sa_family = AF_INET;

 	/* I want IP address attached to "eth0" */
 	strncpy(ifr.ifr_name, interf , IFNAMSIZ-1);

 	ret = ioctl(fd, SIOCGIFADDR, &ifr); 

 	if ( ret != 0 ) {
		if( verbose) printf("%s  - ioctl() returned: %d\n", interf, ret);
		close(fd);
		return(1);
	}

 	close(fd);

 	/* display result */
 	sprintf(IP_interf, "%s:%s", interf, inet_ntoa(((struct sockaddr_in *)&ifr.ifr_addr)->sin_addr));
	oledWriteString(0,line++, IP_interf, FONT_SMALL);

	interf_found++;

 	return(0);
}


// set buttons
void	set_buttons()
{

    	// Intialize the wiringPi Library
    	wiringPiSetup();

	pinMode( INPUT_UP, INPUT );
        pinMode( INPUT_DWN, INPUT );
        pinMode( INPUT_ENT, INPUT );
        pinMode( INPUT_LEFT, INPUT );
        pinMode( INPUT_RIGHT, INPUT );
}

// check buttons

int	check_buttons()
{

  int butt = 0;

	 if(digitalRead(INPUT_UP) == HIGH)   {
			ud = 1;
			butt++;
	 }
	 if(digitalRead(INPUT_DWN) == HIGH)  {
			 ud = -1;
			 butt++;
   	 }
	 if(digitalRead(INPUT_ENT) == HIGH)  {
			 ent = 1;
			 butt++;
   	 }
	 if(digitalRead(INPUT_LEFT) == HIGH) {
			 lr  = -1;
			 butt++;
   	 }
	 if(digitalRead(INPUT_RIGHT) == HIGH) {
			lr = 1;
			butt++;
   	 }

	if( verbose) printf("Butt %d\n",butt);

	return butt;
}





int replace_GPS_Data()
{

int ret=0;

	oledFill(0); // fill with black
	oledWriteString(0,0,"query gpsd ...",FONT_SMALL);
	//if( verbose) printf("now replacing GPS....\n");

	ret = build_APRSconf();

	if ( ret < 0 )  {
		oledWriteString(0,7, "GPS-failure!",  FONT_SMALL);
		sleep(5);
	}

}





void	print_OLED( int line, char *string )
{

	oledWriteString(0,line, string, FONT_SMALL);
}
